# Generated by sgb2hier-ns,sgb2hier-ns generated from sgb2ns
# sgb2ns adapted from sgb2alt by Polly Huang
# GRAPH (#nodes #edges id uu vv ww xx yy zz):
# 100 386 transtub(0,3,0,0,{4,89,3,1.000,0.000,0.000},{1,44,3,0.600,0.000,0.000},{8,46,3,0.420,0.000,0.000}) 89 0 4 1 

#Creating hierarchical topology from transit-stub graph:

proc create-hier-topology {linkBW} {
	global ns n

	set verbose 1

	if {$verbose} { 
		puts "Creating hierarchical nodes.." 
}
set n(0) [$ns node 0.0.0]
	if {$verbose} { puts -nonewline "0..."; flush stdout }
set n(1) [$ns node 0.1.0]
set n(2) [$ns node 0.1.1]
set n(3) [$ns node 0.1.2]
set n(4) [$ns node 0.1.3]
set n(5) [$ns node 0.1.4]
set n(6) [$ns node 0.1.5]
set n(7) [$ns node 0.1.6]
set n(8) [$ns node 0.1.7]
set n(9) [$ns node 0.2.0]
set n(10) [$ns node 0.2.1]
set n(11) [$ns node 0.2.2]
set n(12) [$ns node 0.2.3]
set n(13) [$ns node 0.2.4]
set n(14) [$ns node 0.2.5]
set n(15) [$ns node 0.2.6]
set n(16) [$ns node 0.2.7]
set n(17) [$ns node 0.3.0]
set n(18) [$ns node 0.3.1]
set n(19) [$ns node 0.3.2]
set n(20) [$ns node 0.3.3]
set n(21) [$ns node 0.3.4]
set n(22) [$ns node 0.3.5]
set n(23) [$ns node 0.3.6]
set n(24) [$ns node 0.3.7]
set n(25) [$ns node 1.0.0]
set n(26) [$ns node 1.1.0]
set n(27) [$ns node 1.1.1]
set n(28) [$ns node 1.1.2]
set n(29) [$ns node 1.1.3]
set n(30) [$ns node 1.1.4]
set n(31) [$ns node 1.2.0]
set n(32) [$ns node 1.2.1]
set n(33) [$ns node 1.2.2]
set n(34) [$ns node 1.2.3]
set n(35) [$ns node 1.2.4]
set n(36) [$ns node 1.2.5]
set n(37) [$ns node 1.2.6]
set n(38) [$ns node 1.2.7]
set n(39) [$ns node 1.2.8]
set n(40) [$ns node 1.2.9]
set n(41) [$ns node 1.3.0]
set n(42) [$ns node 1.3.1]
set n(43) [$ns node 1.3.2]
set n(44) [$ns node 1.3.3]
set n(45) [$ns node 1.3.4]
set n(46) [$ns node 1.3.5]
set n(47) [$ns node 1.3.6]
set n(48) [$ns node 1.3.7]
set n(49) [$ns node 1.3.8]
set n(50) [$ns node 2.0.0]
set n(51) [$ns node 2.1.0]
set n(52) [$ns node 2.1.1]
set n(53) [$ns node 2.1.2]
set n(54) [$ns node 2.1.3]
set n(55) [$ns node 2.1.4]
set n(56) [$ns node 2.1.5]
set n(57) [$ns node 2.1.6]
set n(58) [$ns node 2.1.7]
set n(59) [$ns node 2.2.0]
set n(60) [$ns node 2.2.1]
set n(61) [$ns node 2.2.2]
set n(62) [$ns node 2.2.3]
set n(63) [$ns node 2.2.4]
set n(64) [$ns node 2.2.5]
set n(65) [$ns node 2.2.6]
set n(66) [$ns node 2.2.7]
set n(67) [$ns node 2.2.8]
set n(68) [$ns node 2.2.9]
set n(69) [$ns node 2.2.10]
set n(70) [$ns node 2.2.11]
set n(71) [$ns node 2.3.0]
set n(72) [$ns node 2.3.1]
set n(73) [$ns node 2.3.2]
set n(74) [$ns node 2.3.3]
set n(75) [$ns node 3.0.0]
set n(76) [$ns node 3.1.0]
set n(77) [$ns node 3.1.1]
set n(78) [$ns node 3.1.2]
set n(79) [$ns node 3.1.3]
set n(80) [$ns node 3.1.4]
set n(81) [$ns node 3.1.5]
set n(82) [$ns node 3.1.6]
set n(83) [$ns node 3.1.7]
set n(84) [$ns node 3.1.8]
set n(85) [$ns node 3.1.9]
set n(86) [$ns node 3.2.0]
set n(87) [$ns node 3.2.1]
set n(88) [$ns node 3.2.2]
set n(89) [$ns node 3.2.3]
set n(90) [$ns node 3.2.4]
set n(91) [$ns node 3.3.0]
set n(92) [$ns node 3.3.1]
set n(93) [$ns node 3.3.2]
set n(94) [$ns node 3.3.3]
set n(95) [$ns node 3.3.4]
set n(96) [$ns node 3.3.5]
set n(97) [$ns node 3.3.6]
set n(98) [$ns node 3.3.7]
set n(99) [$ns node 3.3.8]


# Topology information :
lappend domain 4
AddrParams set domain_num_ $domain
lappend cluster 4 4 4 4 
AddrParams set cluster_num_ $cluster
lappend eilastlevel 1 8 8 8 1 5 10 9 1 8 12 4 1 10 5 9 
AddrParams set nodes_num_ $eilastlevel


	# EDGES (from-node to-node length a b):
	if {$verbose} { 
		puts "Creating links 0..."
		flush stdout 
	}
	$ns duplex-link-of-interfaces $n(0) $n(75) $linkBW 410ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(50) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(25) $linkBW 170ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(25) $linkBW 170ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(50) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(75) $linkBW 410ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(23) $linkBW 160ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(11) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(0) $n(5) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(1) $n(2) $linkBW 320ms DropTail
	if {$verbose} { puts -nonewline "10..."; flush stdout }
	$ns duplex-link-of-interfaces $n(2) $n(3) $linkBW 500ms DropTail
	$ns duplex-link-of-interfaces $n(2) $n(5) $linkBW 260ms DropTail
	$ns duplex-link-of-interfaces $n(2) $n(7) $linkBW 330ms DropTail
	$ns duplex-link-of-interfaces $n(2) $n(8) $linkBW 60ms DropTail
	$ns duplex-link-of-interfaces $n(3) $n(7) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(4) $n(5) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(5) $n(7) $linkBW 80ms DropTail
	$ns duplex-link-of-interfaces $n(6) $n(7) $linkBW 160ms DropTail
	$ns duplex-link-of-interfaces $n(9) $n(11) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(9) $n(14) $linkBW 370ms DropTail
	if {$verbose} { puts -nonewline "20..."; flush stdout }
	$ns duplex-link-of-interfaces $n(9) $n(15) $linkBW 100ms DropTail
	$ns duplex-link-of-interfaces $n(9) $n(16) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(10) $n(11) $linkBW 240ms DropTail
	$ns duplex-link-of-interfaces $n(10) $n(12) $linkBW 160ms DropTail
	$ns duplex-link-of-interfaces $n(10) $n(14) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(10) $n(15) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(11) $n(12) $linkBW 290ms DropTail
	$ns duplex-link-of-interfaces $n(11) $n(14) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(11) $n(15) $linkBW 150ms DropTail
	$ns duplex-link-of-interfaces $n(11) $n(16) $linkBW 330ms DropTail
	if {$verbose} { puts -nonewline "30..."; flush stdout }
	$ns duplex-link-of-interfaces $n(12) $n(13) $linkBW 360ms DropTail
	$ns duplex-link-of-interfaces $n(12) $n(14) $linkBW 360ms DropTail
	$ns duplex-link-of-interfaces $n(12) $n(15) $linkBW 150ms DropTail
	$ns duplex-link-of-interfaces $n(12) $n(16) $linkBW 270ms DropTail
	$ns duplex-link-of-interfaces $n(13) $n(15) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(13) $n(16) $linkBW 440ms DropTail
	$ns duplex-link-of-interfaces $n(14) $n(15) $linkBW 270ms DropTail
	$ns duplex-link-of-interfaces $n(15) $n(16) $linkBW 260ms DropTail
	$ns duplex-link-of-interfaces $n(17) $n(18) $linkBW 180ms DropTail
	$ns duplex-link-of-interfaces $n(17) $n(20) $linkBW 240ms DropTail
	if {$verbose} { puts -nonewline "40..."; flush stdout }
	$ns duplex-link-of-interfaces $n(17) $n(23) $linkBW 320ms DropTail
	$ns duplex-link-of-interfaces $n(18) $n(19) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(18) $n(21) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(18) $n(23) $linkBW 470ms DropTail
	$ns duplex-link-of-interfaces $n(19) $n(20) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(19) $n(21) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(19) $n(22) $linkBW 270ms DropTail
	$ns duplex-link-of-interfaces $n(20) $n(21) $linkBW 230ms DropTail
	$ns duplex-link-of-interfaces $n(20) $n(22) $linkBW 90ms DropTail
	$ns duplex-link-of-interfaces $n(21) $n(22) $linkBW 170ms DropTail
	if {$verbose} { puts -nonewline "50..."; flush stdout }
	$ns duplex-link-of-interfaces $n(22) $n(23) $linkBW 160ms DropTail
	$ns duplex-link-of-interfaces $n(23) $n(24) $linkBW 340ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(75) $linkBW 450ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(50) $linkBW 430ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(50) $linkBW 430ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(75) $linkBW 450ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(48) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(39) $linkBW 370ms DropTail
	$ns duplex-link-of-interfaces $n(25) $n(30) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(26) $n(27) $linkBW 110ms DropTail
	if {$verbose} { puts -nonewline "60..."; flush stdout }
	$ns duplex-link-of-interfaces $n(26) $n(28) $linkBW 170ms DropTail
	$ns duplex-link-of-interfaces $n(26) $n(29) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(27) $n(28) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(27) $n(30) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(28) $n(30) $linkBW 370ms DropTail
	$ns duplex-link-of-interfaces $n(29) $n(30) $linkBW 100ms DropTail
	$ns duplex-link-of-interfaces $n(31) $n(32) $linkBW 60ms DropTail
	$ns duplex-link-of-interfaces $n(31) $n(35) $linkBW 150ms DropTail
	$ns duplex-link-of-interfaces $n(31) $n(37) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(31) $n(38) $linkBW 90ms DropTail
	if {$verbose} { puts -nonewline "70..."; flush stdout }
	$ns duplex-link-of-interfaces $n(31) $n(39) $linkBW 290ms DropTail
	$ns duplex-link-of-interfaces $n(32) $n(36) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(32) $n(37) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(32) $n(38) $linkBW 70ms DropTail
	$ns duplex-link-of-interfaces $n(33) $n(34) $linkBW 110ms DropTail
	$ns duplex-link-of-interfaces $n(33) $n(37) $linkBW 90ms DropTail
	$ns duplex-link-of-interfaces $n(33) $n(39) $linkBW 90ms DropTail
	$ns duplex-link-of-interfaces $n(33) $n(40) $linkBW 310ms DropTail
	$ns duplex-link-of-interfaces $n(34) $n(35) $linkBW 230ms DropTail
	$ns duplex-link-of-interfaces $n(34) $n(37) $linkBW 90ms DropTail
	if {$verbose} { puts -nonewline "80..."; flush stdout }
	$ns duplex-link-of-interfaces $n(34) $n(40) $linkBW 360ms DropTail
	$ns duplex-link-of-interfaces $n(35) $n(36) $linkBW 290ms DropTail
	$ns duplex-link-of-interfaces $n(35) $n(38) $linkBW 80ms DropTail
	$ns duplex-link-of-interfaces $n(36) $n(39) $linkBW 310ms DropTail
	$ns duplex-link-of-interfaces $n(38) $n(39) $linkBW 320ms DropTail
	$ns duplex-link-of-interfaces $n(39) $n(40) $linkBW 280ms DropTail
	$ns duplex-link-of-interfaces $n(41) $n(42) $linkBW 340ms DropTail
	$ns duplex-link-of-interfaces $n(41) $n(47) $linkBW 60ms DropTail
	$ns duplex-link-of-interfaces $n(41) $n(48) $linkBW 260ms DropTail
	$ns duplex-link-of-interfaces $n(42) $n(46) $linkBW 280ms DropTail
	if {$verbose} { puts -nonewline "90..."; flush stdout }
	$ns duplex-link-of-interfaces $n(42) $n(47) $linkBW 380ms DropTail
	$ns duplex-link-of-interfaces $n(43) $n(44) $linkBW 400ms DropTail
	$ns duplex-link-of-interfaces $n(43) $n(46) $linkBW 420ms DropTail
	$ns duplex-link-of-interfaces $n(44) $n(46) $linkBW 110ms DropTail
	$ns duplex-link-of-interfaces $n(44) $n(48) $linkBW 300ms DropTail
	$ns duplex-link-of-interfaces $n(45) $n(46) $linkBW 390ms DropTail
	$ns duplex-link-of-interfaces $n(45) $n(47) $linkBW 100ms DropTail
	$ns duplex-link-of-interfaces $n(47) $n(49) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(48) $n(49) $linkBW 90ms DropTail
	$ns duplex-link-of-interfaces $n(50) $n(75) $linkBW 120ms DropTail
	if {$verbose} { puts -nonewline "100..."; flush stdout }
	$ns duplex-link-of-interfaces $n(50) $n(75) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(50) $n(71) $linkBW 140ms DropTail
	$ns duplex-link-of-interfaces $n(50) $n(64) $linkBW 360ms DropTail
	$ns duplex-link-of-interfaces $n(50) $n(52) $linkBW 260ms DropTail
	$ns duplex-link-of-interfaces $n(51) $n(55) $linkBW 430ms DropTail
	$ns duplex-link-of-interfaces $n(51) $n(57) $linkBW 180ms DropTail
	$ns duplex-link-of-interfaces $n(52) $n(53) $linkBW 300ms DropTail
	$ns duplex-link-of-interfaces $n(52) $n(55) $linkBW 420ms DropTail
	$ns duplex-link-of-interfaces $n(52) $n(56) $linkBW 60ms DropTail
	$ns duplex-link-of-interfaces $n(53) $n(55) $linkBW 430ms DropTail
	if {$verbose} { puts -nonewline "110..."; flush stdout }
	$ns duplex-link-of-interfaces $n(53) $n(58) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(54) $n(58) $linkBW 170ms DropTail
	$ns duplex-link-of-interfaces $n(55) $n(56) $linkBW 480ms DropTail
	$ns duplex-link-of-interfaces $n(55) $n(58) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(56) $n(58) $linkBW 280ms DropTail
	$ns duplex-link-of-interfaces $n(59) $n(60) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(59) $n(61) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(59) $n(62) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(59) $n(64) $linkBW 310ms DropTail
	$ns duplex-link-of-interfaces $n(59) $n(67) $linkBW 240ms DropTail
	if {$verbose} { puts -nonewline "120..."; flush stdout }
	$ns duplex-link-of-interfaces $n(59) $n(70) $linkBW 40ms DropTail
	$ns duplex-link-of-interfaces $n(60) $n(61) $linkBW 80ms DropTail
	$ns duplex-link-of-interfaces $n(60) $n(62) $linkBW 160ms DropTail
	$ns duplex-link-of-interfaces $n(60) $n(63) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(60) $n(70) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(62) $linkBW 180ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(63) $linkBW 330ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(64) $linkBW 270ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(65) $linkBW 280ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(67) $linkBW 150ms DropTail
	if {$verbose} { puts -nonewline "130..."; flush stdout }
	$ns duplex-link-of-interfaces $n(61) $n(69) $linkBW 400ms DropTail
	$ns duplex-link-of-interfaces $n(61) $n(70) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(62) $n(66) $linkBW 90ms DropTail
	$ns duplex-link-of-interfaces $n(62) $n(69) $linkBW 370ms DropTail
	$ns duplex-link-of-interfaces $n(62) $n(70) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(63) $n(64) $linkBW 420ms DropTail
	$ns duplex-link-of-interfaces $n(63) $n(66) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(63) $n(67) $linkBW 370ms DropTail
	$ns duplex-link-of-interfaces $n(64) $n(65) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(64) $n(66) $linkBW 180ms DropTail
	if {$verbose} { puts -nonewline "140..."; flush stdout }
	$ns duplex-link-of-interfaces $n(64) $n(67) $linkBW 130ms DropTail
	$ns duplex-link-of-interfaces $n(64) $n(68) $linkBW 50ms DropTail
	$ns duplex-link-of-interfaces $n(64) $n(69) $linkBW 440ms DropTail
	$ns duplex-link-of-interfaces $n(65) $n(66) $linkBW 240ms DropTail
	$ns duplex-link-of-interfaces $n(65) $n(68) $linkBW 180ms DropTail
	$ns duplex-link-of-interfaces $n(65) $n(69) $linkBW 230ms DropTail
	$ns duplex-link-of-interfaces $n(65) $n(70) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(66) $n(67) $linkBW 50ms DropTail
	$ns duplex-link-of-interfaces $n(71) $n(72) $linkBW 300ms DropTail
	$ns duplex-link-of-interfaces $n(71) $n(73) $linkBW 210ms DropTail
	if {$verbose} { puts -nonewline "150..."; flush stdout }
	$ns duplex-link-of-interfaces $n(71) $n(74) $linkBW 330ms DropTail
	$ns duplex-link-of-interfaces $n(75) $n(92) $linkBW 200ms DropTail
	$ns duplex-link-of-interfaces $n(75) $n(86) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(75) $n(78) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(76) $n(81) $linkBW 290ms DropTail
	$ns duplex-link-of-interfaces $n(76) $n(82) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(77) $n(78) $linkBW 80ms DropTail
	$ns duplex-link-of-interfaces $n(77) $n(80) $linkBW 300ms DropTail
	$ns duplex-link-of-interfaces $n(77) $n(81) $linkBW 280ms DropTail
	$ns duplex-link-of-interfaces $n(77) $n(82) $linkBW 140ms DropTail
	if {$verbose} { puts -nonewline "160..."; flush stdout }
	$ns duplex-link-of-interfaces $n(77) $n(83) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(77) $n(85) $linkBW 430ms DropTail
	$ns duplex-link-of-interfaces $n(78) $n(82) $linkBW 210ms DropTail
	$ns duplex-link-of-interfaces $n(78) $n(83) $linkBW 330ms DropTail
	$ns duplex-link-of-interfaces $n(78) $n(85) $linkBW 400ms DropTail
	$ns duplex-link-of-interfaces $n(79) $n(82) $linkBW 350ms DropTail
	$ns duplex-link-of-interfaces $n(79) $n(83) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(79) $n(84) $linkBW 100ms DropTail
	$ns duplex-link-of-interfaces $n(81) $n(83) $linkBW 340ms DropTail
	$ns duplex-link-of-interfaces $n(82) $n(85) $linkBW 550ms DropTail
	if {$verbose} { puts -nonewline "170..."; flush stdout }
	$ns duplex-link-of-interfaces $n(83) $n(84) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(84) $n(85) $linkBW 320ms DropTail
	$ns duplex-link-of-interfaces $n(86) $n(90) $linkBW 250ms DropTail
	$ns duplex-link-of-interfaces $n(87) $n(89) $linkBW 120ms DropTail
	$ns duplex-link-of-interfaces $n(88) $n(90) $linkBW 140ms DropTail
	$ns duplex-link-of-interfaces $n(89) $n(90) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(91) $n(92) $linkBW 80ms DropTail
	$ns duplex-link-of-interfaces $n(91) $n(96) $linkBW 220ms DropTail
	$ns duplex-link-of-interfaces $n(91) $n(98) $linkBW 190ms DropTail
	$ns duplex-link-of-interfaces $n(92) $n(93) $linkBW 150ms DropTail
	if {$verbose} { puts -nonewline "180..."; flush stdout }
	$ns duplex-link-of-interfaces $n(92) $n(96) $linkBW 290ms DropTail
	$ns duplex-link-of-interfaces $n(92) $n(98) $linkBW 110ms DropTail
	$ns duplex-link-of-interfaces $n(93) $n(94) $linkBW 230ms DropTail
	$ns duplex-link-of-interfaces $n(93) $n(95) $linkBW 270ms DropTail
	$ns duplex-link-of-interfaces $n(94) $n(95) $linkBW 70ms DropTail
	$ns duplex-link-of-interfaces $n(94) $n(96) $linkBW 400ms DropTail
	$ns duplex-link-of-interfaces $n(94) $n(99) $linkBW 30ms DropTail
	$ns duplex-link-of-interfaces $n(95) $n(96) $linkBW 410ms DropTail
	$ns duplex-link-of-interfaces $n(95) $n(97) $linkBW 370ms DropTail
	$ns duplex-link-of-interfaces $n(96) $n(98) $linkBW 400ms DropTail
	if {$verbose} { puts -nonewline "190..."; flush stdout }
	$ns duplex-link-of-interfaces $n(96) $n(99) $linkBW 410ms DropTail
	$ns duplex-link-of-interfaces $n(97) $n(98) $linkBW 360ms DropTail
	$ns duplex-link-of-interfaces $n(97) $n(99) $linkBW 370ms DropTail

	if {$verbose} { 
		puts -nonewline "193..."
		flush stdout
		puts "starting"
	}
	return 100}
# end of hier topology generation
